@@ -5,8 +5,8 @@
 #
 # Copyright (c) 2007-2008 Peng Huang <shawn.p.huang@gmail.com>
 # Copyright (c) 2009 Hideaki ABE <abe.sendai@gmail.com>
-# Copyright (c) 2010-2013 Takao Fujiwara <takao.fujiwara1@gmail.com>
-# Copyright (c) 2007-2013 Red Hat, Inc.
+# Copyright (c) 2010-2014 Takao Fujiwara <takao.fujiwara1@gmail.com>
+# Copyright (c) 2007-2014 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ class AnthyPrefs(Prefs):
 
         if not self.fetch_item(section, old_key, True):
             return
-        print >> sys.stderr, warning_message
+        print(warning_message, file=sys.stderr)
         if self.fetch_item(section, new_key, True):
             return
 
@@ -69,13 +69,13 @@ class AnthyPrefs(Prefs):
     def keys(self, section):
         if section.startswith('shortcut/'):
             return _cmd_keys
-        return self.default[section].keys()
+        return list(self.default[section].keys())
 
     def get_japanese_ordered_list(self):
         return _japanese_ordered_list
 
     def get_version(self):
-        return '1.5.4'
+        return '1.5.7'
 
     # Convert typing sequences to gsettings key.
     # E.g. 'largea-bracketleft' to 'A['
@@ -88,7 +88,7 @@ class AnthyPrefs(Prefs):
             try:
                 ch = _config_key_to_char[key]
             except KeyError:
-                print >> sys.stderr, 'Not supported key in gsettings', gkeys
+                print('Not supported key in gsettings', gkeys, file=sys.stderr)
                 retval = ''
                 break
             retval += ch
@@ -100,16 +100,9 @@ class AnthyPrefs(Prefs):
         retval = ''
         if self._char_to_config_key == None:
             self._char_to_config_key = {}
-            for _key, _ch in _config_key_to_char.items():
+            for _key, _ch in list(_config_key_to_char.items()):
                 self._char_to_config_key[_ch] = _key
         for ch in typing:
-            try:
-                # U+A5 needs to be UTF-8 since gconf values are
-                # disk saved values.
-                ch = ch.encode('utf-8')
-            except:
-                print >> sys.stderr, \
-                    'Failed to encode UTF-8:', ch
             if ch in _supported_gsettings_key_chars:
                 if retval != '':
                     retval += '-'
@@ -118,7 +111,7 @@ class AnthyPrefs(Prefs):
             try:
                 key = self._char_to_config_key[ch]
             except KeyError:
-                print >> sys.stderr, 'Not supported key in gsettings', typing
+                print('Not supported key in gsettings', typing, file=sys.stderr)
                 retval = ''
                 break
             if retval != '':
@@ -938,135 +931,135 @@ _config = {
     },
 
     'thumb_typing_rule/base': {
-        'q'             : [u'。', u'',   u'ぁ'],
-        'w'             : [u'か', u'が', u'え'],
-        'e'             : [u'た', u'だ', u'り'],
-        'r'             : [u'こ', u'ご', u'ゃ'],
-        't'             : [u'さ', u'ざ', u'れ'],
+        'q'             : ['。', '',   'ぁ'],
+        'w'             : ['か', 'が', 'え'],
+        'e'             : ['た', 'だ', 'り'],
+        'r'             : ['こ', 'ご', 'ゃ'],
+        't'             : ['さ', 'ざ', 'れ'],
 
-        'y'             : [u'ら', u'よ', u'ぱ'],
-        'u'             : [u'ち', u'に', u'ぢ'],
-        'i'             : [u'く', u'る', u'ぐ'],
-        'o'             : [u'つ', u'ま', u'づ'],
-        'p'             : [u'，',  u'ぇ', u'ぴ'],
-        'at'            : [u'、', u'',   u''],
-        'bracketleft'   : [u'゛', u'゜', u''],
+        'y'             : ['ら', 'よ', 'ぱ'],
+        'u'             : ['ち', 'に', 'ぢ'],
+        'i'             : ['く', 'る', 'ぐ'],
+        'o'             : ['つ', 'ま', 'づ'],
+        'p'             : ['，',  'ぇ', 'ぴ'],
+        'at'            : ['、', '',   ''],
+        'bracketleft'   : ['゛', '゜', ''],
 
-        'a'             : [u'う', u'',   u'を'],
-        's'             : [u'し', u'じ', u'あ'],
-        'd'             : [u'て', u'で', u'な'],
-        'f'             : [u'け', u'げ', u'ゅ'],
-        'g'             : [u'せ', u'ぜ', u'も'],
+        'a'             : ['う', '',   'を'],
+        's'             : ['し', 'じ', 'あ'],
+        'd'             : ['て', 'で', 'な'],
+        'f'             : ['け', 'げ', 'ゅ'],
+        'g'             : ['せ', 'ぜ', 'も'],
 
-        'h'             : [u'は', u'み', u'ば'],
-        'j'             : [u'と', u'お', u'ど'],
-        'k'             : [u'き', u'の', u'ぎ'],
-        'l'             : [u'い', u'ょ', u'ぽ'],
-        'semicolon'     : [u'ん', u'っ', u''],
+        'h'             : ['は', 'み', 'ば'],
+        'j'             : ['と', 'お', 'ど'],
+        'k'             : ['き', 'の', 'ぎ'],
+        'l'             : ['い', 'ょ', 'ぽ'],
+        'semicolon'     : ['ん', 'っ', ''],
 
-        'z'             : [u'．',  u'',   u'ぅ'],
-        'x'             : [u'ひ', u'び', u'ー'],
-        'c'             : [u'す', u'ず', u'ろ'],
-        'v'             : [u'ふ', u'ぶ', u'や'],
-        'b'             : [u'へ', u'べ', u'ぃ'],
+        'z'             : ['．',  '',   'ぅ'],
+        'x'             : ['ひ', 'び', 'ー'],
+        'c'             : ['す', 'ず', 'ろ'],
+        'v'             : ['ふ', 'ぶ', 'や'],
+        'b'             : ['へ', 'べ', 'ぃ'],
 
-        'n'             : [u'め', u'ぬ', u'ぷ'],
-        'm'             : [u'そ', u'ゆ', u'ぞ'],
-        'comma'         : [u'ね', u'む', u'ぺ'],
-        'period'        : [u'ほ', u'わ', u'ぼ'],
-        'slash'         : [u'・', u'ぉ', u''],
+        'n'             : ['め', 'ぬ', 'ぷ'],
+        'm'             : ['そ', 'ゆ', 'ぞ'],
+        'comma'         : ['ね', 'む', 'ぺ'],
+        'period'        : ['ほ', 'わ', 'ぼ'],
+        'slash'         : ['・', 'ぉ', ''],
 
-        '1'             : [u'1',  u'',   u'？'],
-        '2'             : [u'2',  u'',   u'／'],
-        '4'             : [u'4',  u'',   u'「'],
-        '5'             : [u'5',  u'',   u'」'],
+        '1'             : ['1',  '',   '？'],
+        '2'             : ['2',  '',   '／'],
+        '4'             : ['4',  '',   '「'],
+        '5'             : ['5',  '',   '」'],
 
-        '6'             : [u'6',  u'［',  u''],
-        '7'             : [u'7',  u'］',  u''],
-        '8'             : [u'8',  u'（',  u''],
-        '9'             : [u'9',  u'）',  u''],
-        'backslash'     : [u'￥', u'',  u''],
+        '6'             : ['6',  '［',  ''],
+        '7'             : ['7',  '］',  ''],
+        '8'             : ['8',  '（',  ''],
+        '9'             : ['9',  '）',  ''],
+        'backslash'     : ['￥', '',  ''],
     },
 
     'thumb_typing_rule/nicola_j_table': {
-        'colon'         : [u'：', u'',   u''],
-        'at'            : [u'、', u'',   u''],
-        'bracketleft'   : [u'゛', u'゜', u''],
-        'bracketright'  : [u'」', u'',   u''],
-        '8'             : [u'8',  u'（', u''],
-        '9'             : [u'9',  u'）', u''],
-        '0'             : [u'0',  u'',   u''],
+        'colon'         : ['：', '',   ''],
+        'at'            : ['、', '',   ''],
+        'bracketleft'   : ['゛', '゜', ''],
+        'bracketright'  : ['」', '',   ''],
+        '8'             : ['8',  '（', ''],
+        '9'             : ['9',  '）', ''],
+        '0'             : ['0',  '',   ''],
     },
 
     'thumb_typing_rule/nicola_a_table': {
-        'colon'         : [u'：', u'',   u''],
-        'at'            : [u'＠', u'',   u''],
-        'bracketleft'   : [u'、', u'',   u''],
-        'bracketright'  : [u'゛', u'゜', u''],
-        '8'             : [u'8',  u'',   u''],
-        '9'             : [u'9',  u'（', u''],
-        '0'             : [u'0',  u'）', u''],
+        'colon'         : ['：', '',   ''],
+        'at'            : ['＠', '',   ''],
+        'bracketleft'   : ['、', '',   ''],
+        'bracketright'  : ['゛', '゜', ''],
+        '8'             : ['8',  '',   ''],
+        '9'             : ['9',  '（', ''],
+        '0'             : ['0',  '）', ''],
     },
 
     'thumb_typing_rule/nicola_f_table': {
-        'colon'         : [u'、', u'',   u''],
-        'at'            : [u'＠', u'',   u''],
-        'bracketleft'   : [u'゛', u'゜', u''],
-        'bracketright'  : [u'」', u'',   u''],
-        '8'             : [u'8',  u'（', u''],
-        '9'             : [u'9',  u'）', u''],
-        '0'             : [u'0',  u'',   u''],
+        'colon'         : ['、', '',   ''],
+        'at'            : ['＠', '',   ''],
+        'bracketleft'   : ['゛', '゜', ''],
+        'bracketright'  : ['」', '',   ''],
+        '8'             : ['8',  '（', ''],
+        '9'             : ['9',  '）', ''],
+        '0'             : ['0',  '',   ''],
     },
 
     'thumb_typing_rule/kb231_j_fmv_table': {
-        '3'             : [u'3',  u'',   u'～'],
-        '0'             : [u'0',  u'『', u''],
-        'minus'         : [u'-',  u'』', u''],
-        'equal'         : [u'=',  u'',   u''],
+        '3'             : ['3',  '',   '～'],
+        '0'             : ['0',  '『', ''],
+        'minus'         : ['-',  '』', ''],
+        'equal'         : ['=',  '',   ''],
     },
 
     'thumb_typing_rule/kb231_a_fmv_table': {
-        '3'             : [u'3',  u'',   u'～'],
-        '0'             : [u'0',  u'）', u''],
-        'minus'         : [u'-',  u'『', u''],
-        'equal'         : [u'=',  u'』', u''],
+        '3'             : ['3',  '',   '～'],
+        '0'             : ['0',  '）', ''],
+        'minus'         : ['-',  '『', ''],
+        'equal'         : ['=',  '』', ''],
     },
 
     'thumb_typing_rule/kb231_f_fmv_table': {
-        '3'             : [u'3',  u'',   u'～'],
-        '0'             : [u'0',  u'『', u''],
-        'minus'         : [u'-',  u'』', u''],
-        'equal'         : [u'=',  u'',   u''],
+        '3'             : ['3',  '',   '～'],
+        '0'             : ['0',  '『', ''],
+        'minus'         : ['-',  '』', ''],
+        'equal'         : ['=',  '',   ''],
     },
 
     'thumb_typing_rule/kb611_j_fmv_table': {
-        'grave'         :  [u'‘', u'',   u''],
-        'asciicircum'   :  [u'々', u'£',  u''],
-        'colon'         :  [u'：', u'',   u''],
-        'at'            :  [u'、', u'¢',  u''],
-        'bracketleft'   :  [u'゛', u'゜', u''],
+        'grave'         :  ['‘', '',   ''],
+        'asciicircum'   :  ['々', '£',  ''],
+        'colon'         :  ['：', '',   ''],
+        'at'            :  ['、', '¢',  ''],
+        'bracketleft'   :  ['゛', '゜', ''],
         # keysyms are same and keycodes depend on the platforms.
         #'￥'            : [u'￥', u'¬',  u''],
-        'backslash'     : [u'￥', u'¦',  u''],
+        'backslash'     : ['￥', '¦',  ''],
     },
 
     'thumb_typing_rule/kb611_a_fmv_table': {
-        'grave'         :  [u'々', u'',   u'£'],
-        'colon'         :  [u'：', u'',   u''],
-        'at'            :  [u'＠', u'',   u''],
-        'bracketleft'   :  [u'、', u'¢',  u''],
+        'grave'         :  ['々', '',   '£'],
+        'colon'         :  ['：', '',   ''],
+        'at'            :  ['＠', '',   ''],
+        'bracketleft'   :  ['、', '¢',  ''],
         #'￥'            : [u'￥', u'¬',  u''],
-        'backslash'     : [u'￥', u'¦',  u''],
+        'backslash'     : ['￥', '¦',  ''],
     },
 
     'thumb_typing_rule/kb611_f_fmv_table': {
-        'grave'         :  [u'‘', u'',   u''],
-        'asciicircum'   :  [u'々', u'£',  u''],
-        'colon'         :  [u'、', u'¢',  u''],
-        'at'            :  [u'＠', u'',   u''],
-        'bracketleft'   :  [u'゛', u'゜', u''],
+        'grave'         :  ['‘', '',   ''],
+        'asciicircum'   :  ['々', '£',  ''],
+        'colon'         :  ['、', '¢',  ''],
+        'at'            :  ['＠', '',   ''],
+        'bracketleft'   :  ['゛', '゜', ''],
         #'￥'            : [u'￥', u'¬',  u''],
-        'backslash'     : [u'￥', u'¦',  u''],
+        'backslash'     : ['￥', '¦',  ''],
     },
 
     'dict': {
@@ -1074,11 +1067,13 @@ _config = {
         'symbol': ['/usr/share/ibus-anthy/dicts/symbol.t'],
         'oldchar': ['/usr/share/ibus-anthy/dicts/oldchar.t'],
         'era': ['/usr/share/ibus-anthy/dicts/era.t'],
+        'emoji': ['/usr/share/ibus-anthy/dicts/emoji.t'],
         'files': [
             '/usr/share/ibus-anthy/dicts/zipcode.t',
             '/usr/share/ibus-anthy/dicts/symbol.t',
             '/usr/share/ibus-anthy/dicts/oldchar.t',
             '/usr/share/ibus-anthy/dicts/era.t',
+            '/usr/share/ibus-anthy/dicts/emoji.t',
         ],
     },
 
@@ -1111,7 +1106,7 @@ _config = {
         'icon': None,
         'short_label': '〒',
         'long_label': N_("Zip Code Conversion"),
-        'preview_lines': 30,
+        'preview_lines': -1,
         'reverse': False,
         'is_system': True,
     },
@@ -1147,6 +1142,16 @@ _config = {
         'reverse': False,
         'is_system': True,
     },
+    'dict/file/emoji': {
+        'embed': False,
+        'single': True,
+        'icon': None,
+        'short_label': '😊',
+        'long_label': N_("Emoji"),
+        'preview_lines': -1,
+        'reverse': False,
+        'is_system': True,
+    },
 }
 
 _shortcut_default = {
