@@ -4,8 +4,8 @@
 #
 # Copyright (c) 2007-2008 Peng Huang <shawn.p.huang@gmail.com>
 # Copyright (c) 2009 Hideaki ABE <abe.sendai@gmail.com>
-# Copyright (c) 2010-2013 Takao Fujiwara <takao.fujiwara1@gmail.com>
-# Copyright (c) 2007-2013 Red Hat, Inc.
+# Copyright (c) 2010-2014 Takao Fujiwara <takao.fujiwara1@gmail.com>
+# Copyright (c) 2007-2014 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,8 +22,11 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 from os import environ, getuid, path
-import os, sys
+import binascii
+import codecs
+import functools
 import locale
+import os, sys
 import xml.dom.minidom
 import gettext
 from gettext import dgettext
@@ -82,22 +85,37 @@ class AnthySetup(object):
         builder.add_from_file(builder_file)
 
         toplevel = builder.get_object('main')
+        parent_xid = 0
+        parent_wmname = None
+        parent_wmclass = None
 
         try:
-            gnome_control_center_xid = int(environ['GNOME_CONTROL_CENTER_XID'])
+            parent_xid = int(environ['IBUS_SETUP_XID'])
+            if parent_xid != 0:
+                parent_wmname = 'ibus-setup'
+                parent_wmclass = 'Ibus-setup'
         except:
-            gnome_control_center_xid = 0
+            pass
 
-        if gnome_control_center_xid != 0:
+        try:
+            if parent_xid == 0:
+                parent_xid = int(environ['GNOME_CONTROL_CENTER_XID'])
+                if parent_xid != 0:
+                    parent_wmname = 'gnome-conrol-center'
+                    parent_wmclass = 'Gnome-conrol-center'
+        except:
+            pass
+
+        if parent_xid != 0:
             def set_transient(obj, pspec):
                 window = toplevel.get_window()
                 if window == None:
                     return
                 parent_window = GdkX11.X11Window.foreign_new_for_display(Gdk.Display.get_default(),
-                                                                         gnome_control_center_xid)
+                                                                         parent_xid)
                 if parent_window != None:
                     window.set_transient_for(parent_window)
-            toplevel.set_wmclass('gnome-control-center', 'Gnome-control-center')
+            toplevel.set_wmclass(parent_wmname, parent_wmclass)
             toplevel.set_modal(True)
             toplevel.set_type_hint(Gdk.WindowTypeHint.DIALOG)
             toplevel.connect('notify::window', set_transient)
@@ -293,9 +311,9 @@ class AnthySetup(object):
                 try:
                     pixbuf = icon_theme.load_icon(icon_path, 48, 0)
                     about_dialog.set_logo(pixbuf)
-                except Exception, err:
-                    print >> sys.stderr, 'Not found icon', str(err)
-                    print >> sys.stderr, 'Need to run gtk-update-icon-cache'
+                except Exception as err:
+                    print('Not found icon', str(err), file=sys.stderr)
+                    print('Need to run gtk-update-icon-cache', file=sys.stderr)
         content_area = about_dialog.get_content_area()
         list = content_area.get_children()
         vbox = list[0]
@@ -325,17 +343,18 @@ class AnthySetup(object):
         return (section, key)
 
     def __run_message_dialog(self, message, type=Gtk.MessageType.INFO):
-        dlg = Gtk.MessageDialog(parent=self.__builder.get_object('main'),
-                                flags=Gtk.DialogFlags.MODAL,
-                                message_type=type,
-                                buttons=Gtk.ButtonsType.OK,
-                                message_format=message)
+        dlg = Gtk.MessageDialog(
+                transient_for=self.__builder.get_object('main'),
+                message_type=type,
+                buttons=Gtk.ButtonsType.OK,
+                text=message)
         dlg.run()
         dlg.destroy()
 
     def __japanese_tuple_sort(self, a, b):
         if a[1] == b[1]:
-            return cmp(a[0], b[0])
+            # http://docs.python.org/3.0/whatsnew/3.0.html#ordering-comparisons
+            return (a[0] > b[0]) - (a[0] < b[0])
         elif a[1] in self.__japanese_ordered_dict and \
             b[1] in self.__japanese_ordered_dict:
             return self.__japanese_ordered_dict[a[1]] - \
@@ -347,10 +366,11 @@ class AnthySetup(object):
             b[1] not in self.__japanese_ordered_dict:
             return -1
         else:
-            return cmp(a[1], b[1])
+            return (a[1] > b[1]) - (a[1] < b[1])
 
     def __japanese_thumb_sort(self, a, b):
-        return cmp(a[0], b[0])
+        # http://docs.python.org/3.0/whatsnew/3.0.html#ordering-comparisons
+        return (a[0] > b[0]) - (a[0] < b[0])
 
     def __renderer_toggled_cb(self, renderer, path, model):
         prefs = self.prefs
@@ -417,9 +437,10 @@ class AnthySetup(object):
             # config.set_value(key, None) is not supported.
             if value != None and value != '':
                 rule[ch] = prefs.str(value)
-        for key, value in sorted(rule.items(), \
-            cmp = self.__japanese_tuple_sort):
+        for key, value in sorted(list(rule.items()), \
+            key = functools.cmp_to_key(self.__japanese_tuple_sort)):
             ls.append(['romaji', key, value])
+        tv.set_model(None)
         tv.append_column(Gtk.TreeViewColumn(_(_("Input Chars")),
                                             Gtk.CellRendererText(), text=1))
         tv.append_column(Gtk.TreeViewColumn(_(_("Output Chars")),
@@ -452,9 +473,10 @@ class AnthySetup(object):
             # config.set_value(key, None) is not supported.
             if value != None and value != '':
                 rule[ch] = prefs.str(value)
-        for key, value in sorted(rule.items(), \
-            cmp = self.__japanese_tuple_sort):
+        for key, value in sorted(list(rule.items()), \
+            key = functools.cmp_to_key(self.__japanese_tuple_sort)):
             ls.append(['kana', key, value])
+        tv.set_model(None)
         tv.append_column(Gtk.TreeViewColumn(_(_("Input Chars")),
                                             Gtk.CellRendererText(), text=1))
         tv.append_column(Gtk.TreeViewColumn(_(_("Output Chars")),
@@ -499,9 +521,10 @@ class AnthySetup(object):
                 rule[ch][0] = prefs.str(value[0])
                 rule[ch][1] = prefs.str(value[1])
                 rule[ch][2] = prefs.str(value[2])
-        for key, value in sorted(rule.items(), \
-            cmp = self.__japanese_thumb_sort):
+        for key, value in sorted(list(rule.items()), \
+            key = functools.cmp_to_key(self.__japanese_thumb_sort)):
             ls.append(['thumb', key, value[0], value[2], value[1]])
+        tv.set_model(None)
         tv.append_column(Gtk.TreeViewColumn(_(_("Input")),
                                             Gtk.CellRendererText(), text=1))
         tv.append_column(Gtk.TreeViewColumn(_(_("Single")),
@@ -582,9 +605,10 @@ class AnthySetup(object):
         combobox.clear()
         combobox.disconnect_by_func(self.on_cb_custom_key_table_changed)
 
-    def __run_dialog_custom_key_table(self, mode):
+    def __run_dialog_custom_key_table(self, widget, mode):
         prefs = self.prefs
         dlg = self.__builder.get_object('dialog_custom_key_table')
+        dlg.set_transient_for(widget.get_toplevel())
         label = self.__builder.get_object('label_custom_key_table')
         label_output = self.__builder.get_object('label_output_custom_key')
         list_labels = []
@@ -683,12 +707,12 @@ class AnthySetup(object):
             id = file
             has_mbcs = False
 
-            for i in xrange(0, len(id)):
+            for i in range(0, len(id)):
                 if ord(id[i]) >= 0x7f:
                     has_mbcs = True
                     break
             if has_mbcs:
-                id = id.encode('hex')
+                id = str(binascii.hexlify(id.encode()), 'ascii')
 
             if id.find('/') >=0:
                 id = id[id.rindex('/') + 1:]
@@ -696,7 +720,7 @@ class AnthySetup(object):
                 id = id[:id.rindex('.')]
 
             if id.startswith('0x'):
-                id = id.encode('hex')
+                id = str(binascii.hexlify(id.encode()), 'ascii')
                 has_mbcs = True
             if has_mbcs:
                 id = '0x' + id
@@ -785,8 +809,8 @@ class AnthySetup(object):
         embed = self.__builder.get_object('dict:embed').get_active()
         reverse = self.__builder.get_object('dict:reverse').get_active()
         short_label = self.__builder.get_object('dict:short_entry').get_text()
-        if len(unicode(short_label, 'utf-8')) > 1:
-            short_label = unicode(short_label, 'utf-8')[0].encode('utf-8')
+        if len(short_label) > 1:
+            short_label = short_label[0]
         long_label = self.__builder.get_object('dict:long_entry').get_text()
 
         if new:
@@ -834,8 +858,8 @@ class AnthySetup(object):
         single = prefs.get_value(section, 'single')
         reverse = prefs.get_value(section, 'reverse')
 
-        if len(prefs.unicode(short_label)) > 1:
-            short_label = prefs.unicode(short_label)[0].encode('utf-8')
+        if len(short_label) > 1:
+                short_label = short_label[0]
         self.__builder.get_object('dict:single').set_active(single)
         self.__builder.get_object('dict:embed').set_active(embed)
         self.__builder.get_object('dict:reverse').set_active(reverse)
@@ -934,8 +958,6 @@ class AnthySetup(object):
                 nodes = elements[0].childNodes
             if len(nodes) > 0:
                 version_str = nodes[0].data
-            if type(version_str) == unicode:
-                version_str = str(version_str)
             if version_str != '':
                 version_str = version_str.strip()
             return version_str
@@ -974,22 +996,16 @@ class AnthySetup(object):
         nodes = dom.getElementsByTagName('layout')[0].childNodes
         if len(nodes) > 0:
             layout = nodes[0].data
-        if type(layout) == unicode:
-            layout = str(layout)
         if layout != '':
             keymap = layout.strip()
         nodes = dom.getElementsByTagName('layout_variant')[0].childNodes
         if len(nodes) > 0:
             variant = nodes[0].data
-        if type(variant) == unicode:
-            variat = str(variant)
         if variant != '':
             keymap += '(' + varaint.strip() + ')'
         nodes = dom.getElementsByTagName('layout_option')[0].childNodes
         if len(nodes) > 0:
             option = nodes[0].data
-        if type(option) == unicode:
-            option = str(option)
         if option != '':
             keymap += '[' + option.strip() + ']'
         return keymap
@@ -1009,7 +1025,7 @@ class AnthySetup(object):
                 variant = keymap[0:length]
                 keymap = keymap[length + 1:]
             else:
-                print >> sys.stderr, 'Invalid keymap', keymap
+                print('Invalid keymap', keymap, file=sys.stderr)
                 return ('', '', '')
         length = keymap.find('[')
         if length >= 0:
@@ -1021,7 +1037,7 @@ class AnthySetup(object):
                 option = keymap[0:length]
                 keymap = keymap[length + 1:]
             else:
-                print >> sys.stderr, 'Invalid keymap', keymap
+                print('Invalid keymap', keymap, file=sys.stderr)
                 return ('', '', '')
 
         if layout == None:
@@ -1058,19 +1074,16 @@ class AnthySetup(object):
         else:
             nodes[0].data = option
         nodes = dom.getElementsByTagName('symbol')[0].childNodes
-        # unicode will causes UnicodeEncodeError in write stream.
-        if len(nodes) > 0 and type(nodes[0].data) == unicode:
-            nodes[0].data = nodes[0].data.encode('utf-8')
-
         user_config = path.join(self.__get_userhome(), '.config',
                                 'ibus-anthy', 'engines.xml')
         dir = path.dirname(user_config)
         if not path.exists(dir):
-            os.makedirs(dir, 0700)
-        f = open(user_config, 'w')
+            os.makedirs(dir, 0o700)
+        # io.open() causes TypeError for unicode.
+        f = codecs.open(user_config, 'w', 'utf-8')
         dom.writexml(f, '', '', '', 'utf-8')
         f.close()
-        os.chmod(user_config, 0600)
+        os.chmod(user_config, 0o600)
         self.__keymap = None
         self.__run_message_dialog(_("Anthy keyboard layout is changed. "
                                     "Please restart ibus to reload the layout."))
@@ -1108,6 +1121,72 @@ class AnthySetup(object):
         if self.__keymap != None:
             self.__save_keymap()
 
+    def __search_and_mark(self, buffer, text, start, end, onetime, forward):
+        if forward:
+            match = start.forward_search(text, 0, end)
+        else:
+            match = start.backward_search(text, 0, end)
+        if match == None:
+            return False
+        match_start, match_end = match
+        if onetime:
+            buffer.place_cursor(match_start)
+            buffer.select_range(match_start, match_end)
+            return True
+        buffer.apply_tag(buffer.tag_found, match_start, match_end)
+        self.__search_and_mark(buffer, text, match_end, end, onetime, forward)
+        return True
+
+    def __filter_search(self, entry, onetime, forward):
+        text = entry.get_text()
+        self.__filter_timeout_id = 0
+
+        text_view = entry.text_view
+        buffer = text_view.get_buffer()
+        start = buffer.get_start_iter()
+        if onetime:
+            bounds = buffer.get_selection_bounds()
+            if len(bounds) != 0:
+                start, end = bounds
+                if forward:
+                    start = end
+        end = buffer.get_end_iter()
+        if not forward:
+            end = buffer.get_start_iter()
+        if not onetime:
+            buffer.remove_all_tags(start, end)
+        if text == '':
+            return
+        found = self.__search_and_mark(buffer, text, start, end, onetime, forward)
+        if not found and onetime and forward:
+            end = start
+            start = buffer.get_start_iter()
+            self.__search_and_mark(buffer, text, start, end, onetime, forward)
+
+    def __do_filter(self, entry):
+        self.__filter_search(entry, False, True)
+        return False
+
+    def __filter_changed(self, entry):
+        if self.__filter_timeout_id != 0:
+            return
+        self.__filter_timeout_id = GLib.timeout_add(150,
+                                                    self.__do_filter,
+                                                    entry)
+
+    def __filter_key_release_event(self, entry, event):
+        pressed, keyval = event.get_keyval()
+        if keyval == IBus.KEY_Return:
+            forward = True
+            if event.get_state() & Gdk.ModifierType.SHIFT_MASK:
+                forward = False
+            self.__filter_search(entry, True, forward)
+            text_view = entry.text_view
+            buffer = text_view.get_buffer()
+            text_view.scroll_to_mark(buffer.get_insert(),
+                                     0.25, False, 0.0, 0.0)
+        return False
+
     def on_selection_changed(self, widget, id):
         set_sensitive = lambda a, b: self.__builder.get_object(a).set_sensitive(b)
         flg = True if widget.get_selected()[1] else False
@@ -1134,6 +1213,7 @@ class AnthySetup(object):
             Gtk.main_quit()
             return True
         dlg = self.__builder.get_object('quit_check')
+        dlg.set_transient_for(widget.get_toplevel())
         dlg.set_markup('<big><b>%s</b></big>' % _("Confirmation"))
         dlg.format_secondary_text(
                 _("You are about to close the setup dialog, is that OK?"))
@@ -1150,6 +1230,7 @@ class AnthySetup(object):
             Gtk.main_quit()
             return True
         dlg = self.__builder.get_object('quit_check_without_save')
+        dlg.set_transient_for(widget.get_toplevel())
         dlg.set_markup('<big><b>%s</b></big>' % _("Notice!"))
         dlg.format_secondary_text(
                 _("You are about to close the setup dialog without saving your changes, is that OK?"))
@@ -1225,6 +1306,7 @@ class AnthySetup(object):
         for w in ['es:checkbutton_ctrl', 'es:checkbutton_alt', 'es:checkbutton_shift']:
             self.__builder.get_object(w).set_active(False)
         dlg = self.__builder.get_object('edit_shortcut')
+        dlg.set_transient_for(widget.get_toplevel())
         id = dlg.run()
         dlg.hide()
         if id == Gtk.ResponseType.OK:
@@ -1245,13 +1327,13 @@ class AnthySetup(object):
             self.__builder.get_object('btn_apply').set_sensitive(True)
 
     def on_btn_romaji_custom_table_clicked(self, widget):
-        self.__run_dialog_custom_key_table('romaji')
+        self.__run_dialog_custom_key_table(widget, 'romaji')
 
     def on_btn_kana_custom_table_clicked(self, widget):
-        self.__run_dialog_custom_key_table('kana')
+        self.__run_dialog_custom_key_table(widget, 'kana')
 
     def on_btn_thumb_custom_table_clicked(self, widget):
-        self.__run_dialog_custom_key_table('thumb')
+        self.__run_dialog_custom_key_table(widget, 'thumb')
 
     def on_btn_add_custom_key(self, widget, user_data):
         prefs = self.prefs
@@ -1380,6 +1462,7 @@ class AnthySetup(object):
         for w in ['es:checkbutton_ctrl', 'es:checkbutton_alt', 'es:checkbutton_shift']:
             self.__builder.get_object(w).set_active(False)
         dlg = self.__builder.get_object('edit_shortcut')
+        dlg.set_transient_for(widget.get_toplevel())
         id = dlg.run()
         dlg.hide()
         self.__builder.get_object('es:button_add').show()
@@ -1414,16 +1497,17 @@ class AnthySetup(object):
 
         if Gtk.Buildable.get_name(widget) == 'dict:btn_add':
             dlg = Gtk.FileChooserDialog(title=_("Open Dictionary File"),
-                                        parent=self.__builder.get_object('main'),
-                                        action=Gtk.FileChooserAction.OPEN,
-                                        buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                                                 Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
+                                        transient_for=widget.get_toplevel(),
+                                        action=Gtk.FileChooserAction.OPEN)
+            buttons=(_("_Cancel"), Gtk.ResponseType.CANCEL,
+                     _("_Open"), Gtk.ResponseType.OK)
+            dlg.add_buttons(*buttons)
         if Gtk.Buildable.get_name(widget) == 'dict:btn_edit':
             dlg = Gtk.Dialog(title=_("Edit Dictionary File"),
-                             parent=self.__builder.get_object('main'),
-                             flags=Gtk.DialogFlags.MODAL,
-                             buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                                      Gtk.STOCK_OK, Gtk.ResponseType.OK))
+                             transient_for=widget.get_toplevel())
+            buttons=(_("_Cancel"), Gtk.ResponseType.CANCEL,
+                     _("_OK"), Gtk.ResponseType.OK)
+            dlg.add_buttons(*buttons)
 
         vbox = self.__builder.get_object('dict:add_extra_vbox')
         if Gtk.Buildable.get_name(widget) == 'dict:btn_add':
@@ -1508,19 +1592,20 @@ class AnthySetup(object):
         encoding = self.prefs.get_value(section, 'encoding')
 
         lines = '';
-        for i, line in enumerate(file(dict_file)):
+        for i, line in enumerate(list(codecs.open(dict_file, 'r', 'utf-8'))):
             if nline >= 0 and i >= nline:
                 break;
             lines = lines + line
         if encoding != None and encoding != 'utf-8':
-            lines = unicode(lines, encoding).encode('utf-8')
+            lines = str(lines, encoding)
 
         dlg = Gtk.Dialog(title=_("View Dictionary File"),
-                         parent=self.__builder.get_object('main'),
-                         flags=Gtk.DialogFlags.MODAL,
-                         buttons=(Gtk.STOCK_OK, Gtk.ResponseType.OK))
+                         transient_for=widget.get_toplevel())
+        buttons=(_("_OK"), Gtk.ResponseType.OK)
+        dlg.add_buttons(*buttons)
         buffer = Gtk.TextBuffer()
         buffer.set_text (lines)
+        buffer.tag_found = buffer.create_tag('found', background = 'yellow')
         text_view = Gtk.TextView.new_with_buffer(buffer)
         text_view.set_editable(False)
         sw = Gtk.ScrolledWindow()
@@ -1530,6 +1615,24 @@ class AnthySetup(object):
         parent_vbox.add(sw)
         sw.show_all()
         dlg.set_default_size(500, 500)
+        self.__filter_timeout_id = 0
+
+        if hasattr(Gtk, 'SearchEntry') and \
+           hasattr(Gtk, 'SearchEntryClass') and \
+           hasattr(Gtk.SearchEntryClass, 'search_changed'):
+            filter_entry = Gtk.SearchEntry(hexpand = True,
+                                           margin_left = 6,
+                                           margin_right = 6,
+                                           margin_top = 6,
+                                           margin_bottom = 6)
+            filter_entry.text_view = text_view
+            filter_entry.connect('search-changed', self.__filter_changed)
+            filter_entry.connect('key-release-event',
+                                 self.__filter_key_release_event)
+            parent_vbox.add(filter_entry)
+            filter_entry.show_all()
+
+        sw.show_all()
         dlg.run()
         dlg.destroy()
 
@@ -1597,14 +1700,14 @@ class AnthySetup(object):
 
     def on_shortcut_key_release_event(self, widget, event):
         if event.hardware_keycode in [36, 65]:
-            self.on_btn_edit_clicked(None)
+            self.on_btn_edit_clicked(widget)
 
     def on_shortcut_click_event(self, widget, event):
         if event.type == Gdk.EventType._2BUTTON_PRESS:
             widget.dc = True
         elif event.type == Gdk.EventType.BUTTON_RELEASE:
             if hasattr(widget, 'dc') and widget.dc:
-                self.on_btn_edit_clicked(None)
+                self.on_btn_edit_clicked(widget)
                 widget.dc = False
 
     def on_key_input_dialog_key_press_event(self, widget, event):
@@ -1666,6 +1769,7 @@ class AnthySetup(object):
 
     def on_es_button_run_input_clicked(self, widget):
         dlg = self.__builder.get_object('key_input_dialog')
+        dlg.set_transient_for(widget.get_toplevel())
         dlg.set_markup('<big><b>%s</b></big>' % _("Please press a key (or a key combination)"))
         dlg.format_secondary_text(_("The dialog will be closed when the key is released"))
         id = dlg.run()
@@ -1686,6 +1790,7 @@ class AnthySetup(object):
         s = self.__builder.get_object('es:entry').get_text()
         if not s or not IBus.keyval_from_name(s):
             dlg = self.__builder.get_object('invalid_keysym')
+            dlg.set_transient_for(widget.get_toplevel())
             dlg.set_markup('<big><b>%s</b></big>' % _("Invalid keysym"))
             dlg.format_secondary_text(_("This keysym is not valid"))
             dlg.run()
@@ -1706,6 +1811,7 @@ class AnthySetup(object):
         s = self.__builder.get_object('es:entry').get_text()
         if not s or not IBus.keyval_from_name(s):
             dlg = self.__builder.get_object('invalid_keysym')
+            dlg.set_transient_for(widget.get_toplevel())
             dlg.set_markup('<big><b>%s</b></big>' % _("Invalid keysym"))
             dlg.format_secondary_text(_("This keysym is not valid"))
             dlg.run()
